# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class ChordOmission(AbjadObject):
    '''A chord ommission.

    Value object that can not be chnaged after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self):
        pass
