p.rubric {
    margin-top: 0;
    font-weight: 700;
    font-family: "Roboto Slab","ff-tisa-web-pro","Georgia",Arial,sans-serif;
    font-size: 150%;
    }

div.example,
p.graphviz {
    -moz-border-radius: 2px;
    -moz-box-shadow: 0px 0px 10px #eee;
    -webkit-border-radius: 2px;
    -webkit-box-shadow: 0px 0px 10px #eee;
    border-radius: 2px;
    border: 1px solid #ddd;
    box-shadow: 0px 0px 10px #eee;
    margin-bottom: 24px;
    padding: 24px;
    }

p.graphviz {
    background-color: white;
    }

div.example :last-child {
    margin-bottom: 0;
    }

tt.attribute {
    color: #fff;
    font-size: 0.75em;
    margin: 0 1em 0 0;
    padding: 0.5em;
    }

tt.abstract {
    background-color: #666;
    }

tt.classmethod {
    background-color: #c96;
    }

tt.inherited {
    background-color: #6cc;
    }

tt.staticmethod {
    background-color: #69c;
    }

.table-row {
    display: table-row;
    }

.table-row :first-child {
    padding-left: 0px;
    }

.table-row :last-child {
    padding-right: 0px;
    }

.table-cell {
    display: table-cell;
    padding-left: 12px;
    padding-right: 12px;
    padding-bottom: 24px;
    }

a.table-cell img {
    -moz-border-radius: 2px;
    -moz-box-shadow: 0px 0px 10px #eee;
    -webkit-border-radius: 2px;
    -webkit-box-shadow: 0px 0px 10px #eee;
    border-radius: 2px;
    border: 1px solid #ddd;
    box-shadow: 0px 0px 10px #eee;
    background-color: white;
    }

a.thumbnail {
    cursor: zoom-in;
    }

a.abjadbook {
    display: block;
    margin: 1px 0 24px 0;
    padding: 0px;
    }

.expression {
    background-color: #eff5ff;
    }
