# -*- coding: utf-8 -*-
import abjad


metadata = abjad.datastructuretools.TypedOrderedDict(
    [
        ('composer', {composer_last_name!r}),
        ('title', {score_title!r}),
        ('year', {year}),
        ]
    )
